# -*- coding: utf-8 -*-
#Copyright (C) 2011 Seán Hayes
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.db import models
from datetime import datetime

class PostManager(models.Manager):
	def active(self):
		return self.filter(status=self.model.PUBLISHED_STATUS, published__lt=datetime.now())



# -*- coding: utf-8 -*-
#Copyright (C) 2011 Seán Hayes
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.db import models
from django.contrib.auth.models import User
from django.contrib.sites.models import Site
from datetime import datetime
from managers import PostManager

# Create your models here.

#http://djangoadvent.com/1.2/object-permissions/

#TODO: add pages (or use Flatpages), custom menus
#Use http://code.google.com/p/django-trackback/

class Channel(models.Model):
	#TODO: add allowed authors, text descriptions
	slug = models.SlugField()
	name = models.CharField(max_length=140, unique=True)
	site = models.ForeignKey(Site)
	
	def __unicode__(self):
		return self.name
	
	class Meta:
		ordering = ['name']
		unique_together = (('slug', 'site'), ('name', 'site'))

class Post(models.Model):
	SUMMARY_LENGTH = 50
	
	DRAFT_STATUS = 1
	PUBLISHED_STATUS = 2
	STATUS_CHOICES = (
		(DRAFT_STATUS, 'Draft',),
		(PUBLISHED_STATUS, 'Published',),
	)
	
	author = models.ForeignKey(User)
	status = models.IntegerField(max_length=1, default=DRAFT_STATUS, choices=STATUS_CHOICES)
	slug = models.SlugField(unique_for_date='published')
	#microblog compatible.
	title = models.CharField(max_length=140, unique_for_date='published')
	text = models.TextField(default="")
	custom_summary = models.TextField(default="")
	channels = models.ManyToManyField(Channel)
	#TODO: add tags
	
	created = models.DateTimeField(auto_now_add=True, editable=False)
	last_edited = models.DateTimeField(auto_now=True, editable=False)
	published = models.DateTimeField(default=datetime.now())#FIXME: should be set on save
	
	def _get_teaser(self):
		"A small excerpt of text that can be used in the absence of a custom summary."
		return self.text[:Post.SUMMARY_LENGTH]
	teaser = property(_get_teaser)
	
	def _get_summary(self):
		"Returns custom_summary, or teaser if not available."
		if len(self.custom_summary) > 0:
			return self.custom_summary
		else:
			return self.teaser
	summary = property(_get_summary)
	
	objects = PostManager()
	
	def __unicode__(self):
		return self.title
	
	class Meta:
		ordering = ['published']

class Item(models.Model):
	#http://www.oembed.com/
	#needs to handle:
	#links to web pages - show screenshot thumbnail
	#oembed-able links - store JSON, render oembed data on request. store thumbnail if none is provided.
	#attachments - generate thumbnail and store if possible, otherwise show generic icon. Embed player/viewer if possible, else give download link
	
	#I think I'll just store title and url, and leave the rest to jquery-oembed.
	#attachments can be handled by creating an oembed provider, or it could be faked.
	#services: http://api.embed.ly/
	post = models.ForeignKey(Post)
	text = models.TextField(default="")
	url = models.URLField(unique=True)
	title = models.CharField(max_length=140)
	order = models.PositiveIntegerField(default=0)
	
	def __unicode__(self):
		return self.title
	
	class Meta:
		ordering = ['order']




# -*- coding: utf-8 -*-
#Copyright (C) 2011 Seán Hayes
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.

from datetime import datetime, timedelta
from django.test import TestCase
from django.contrib.auth.models import User
from django.contrib.sites.models import Site
from django.contrib.comments.models import Comment
from ..models import *
from django.core.cache import cache

class BaseTestCase(TestCase):
	def setUp(self):
		self.username = 'test_user'
		self.password = 'foobar'
		self.user = User.objects.create_user(self.username, 'test_user@example.com', self.password)
		self.c1 = Channel(
			slug='public',
			name='Public',
			site=Site.objects.get_current()
		)
		self.c2 = Channel(
			slug='another-channel',
			name='Another Channel',
			site=Site.objects.get_current()
		)
		self.p1 = Post(
			author=self.user,
			slug='unit-testing-unit-tests',
			title='Are you unit testing your unit tests? Learn all about the latest best practice: TDTDD',
			text='Lorem Ipsum etc.',
			status=Post.PUBLISHED_STATUS
		)
		self.p2 = Post(
			author=self.user,
			slug='tree-falls-forest',
			title='Tree Falls in Forest, No One Notices',
			published=datetime.now()+timedelta(days=1),
			status=Post.PUBLISHED_STATUS
		)
		self.p3 = Post(
			author=self.user,
			slug='tree-falls-forest-again',
			title='Tree Falls in Forest, Could There be a Tree Flu Epidemic?',
			status=Post.DRAFT_STATUS
		)
		self.i1 = Item(
			text='A Really Awesome Site',
			url='example.com',
			title='Example.com Homepage',
		)
		self.i2 = Item(
			text='Another Really Awesome Site',
			url='test.com',
			title='Test.com Homepage',
		)
		self.comment1 = Comment(
			site=Site.objects.get_current(),
			user=self.user,
			comment='Thanks for sharing.',
		
		)
	
	def tearDown(self):
		#FIXME: dqc doesn't intercept db destruction or rollback
		cache.clear()



# -*- coding: utf-8 -*-
#Copyright (C) 2011 Seán Hayes
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.views.generic import list_detail
from django.http import HttpResponseRedirect
from django.template import RequestContext
from django.core.urlresolvers import reverse
from django.shortcuts import render_to_response
from models import *
import logging

logger = logging.getLogger(__name__)

# Create your views here.

def index(request):
	return list_detail.object_list(
		request,
		queryset = Post.objects.active(),
		template_name='django_mesh/index.html',
		template_object_name='post'
	)

def channel_index(request):
	#FIXME: only show channels for this site
	return list_detail.object_list(
		request,
		queryset = Channel.objects.all(),
		template_name='django_mesh/channel_index.html',
		template_object_name='channel'
	)

def channel_view(request, slug):
	c = Channel.objects.get(slug=slug)
	return list_detail.object_list(
		request,
		queryset = Post.objects.active().filter(channels=c),
		template_name='django_mesh/channel_view.html',
		template_object_name='post'
	)

def post_index(request):
	return list_detail.object_list(
		request,
		queryset = Post.objects.active(),
		template_name='django_mesh/post_index.html',
		template_object_name='post'
	)

def post_view(request, slug):
	return list_detail.object_detail(
		request,
		Post.objects.active(),
		slug=slug,
		template_name='django_mesh/post_view.html',
		template_object_name='post'
	)

def post_comments(request, slug):
	return list_detail.object_detail(
		request,
		Post.objects.active(),
		slug=slug,
		template_name='django_mesh/post_comments.html',
		template_object_name='post'
	)



# -*- coding: utf-8 -*-
#Copyright (C) 2011 Seán Hayes
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.

from util import BaseTestCase
from django.core.urlresolvers import reverse
import pdb

class IndexTestCase(BaseTestCase):
	
	def test_empty(self):
		response = self.client.get(reverse('mesh_index'))
		
		self.assertEqual(response.status_code, 200)
		self.assertContains(response, "There are no posts to display.")
	
	def test_has_only_active_posts(self):
		self.c1.save()
		self.p1.save()
		self.p1.channels.add(self.c1)
		self.p1.save()
		self.p2.save()
		self.p2.channels.add(self.c1)
		self.p2.save()
		self.p3.save()
		self.p3.channels.add(self.c1)
		self.p3.save()
		
		response = self.client.get(reverse('mesh_index'))
		
		self.assertEqual(response.status_code, 200)
		self.assertContains(response, self.p1.title)
		self.assertNotContains(response, self.p2.title)
		self.assertNotContains(response, self.p3.title)

class ChannelTestCase(BaseTestCase):
	
	def test_index_empty(self):
		response = self.client.get(reverse('mesh_channel_index'))
		
		self.assertEqual(response.status_code, 200)
		self.assertContains(response, "There are no channels to display.")
	
	def test_index_non_empty(self):
		self.c1.save()
		
		response = self.client.get(reverse('mesh_channel_index'))
		
		self.assertEqual(response.status_code, 200)
		self.assertContains(response, self.c1.name)
	
	def test_view_empty(self):
		self.c1.save()
		
		response = self.client.get(reverse('mesh_channel_view', args=[self.c1.slug]))
		
		self.assertEqual(response.status_code, 200)
		self.assertContains(response, "There are no posts to display.")
	
	def test_view_has_no_posts_from_other_channels(self):
		self.c1.save()
		
		self.p1.save()
		self.p1.channels.add(self.c1)
		self.p1.save()
		
		self.c2.save()
		
		self.p2.save()
		self.p2.channels.add(self.c2)
		self.p2.save()
		
		response = self.client.get(reverse('mesh_channel_view', args=[self.c1.slug]))
		
		self.assertEqual(response.status_code, 200)
		self.assertContains(response, self.p1.title)
		self.assertNotContains(response, self.p2.title)
	
	def test_view_has_only_active_posts(self):
		self.c1.save()
		self.p1.save()
		self.p1.channels.add(self.c1)
		self.p1.save()
		self.p2.save()
		self.p2.channels.add(self.c1)
		self.p2.save()
		self.p3.save()
		self.p3.channels.add(self.c1)
		self.p3.save()
		
		response = self.client.get(reverse('mesh_channel_view', args=[self.c1.slug]))
		
		self.assertEqual(response.status_code, 200)
		self.assertContains(response, self.p1.title)
		self.assertNotContains(response, self.p2.title)
		self.assertNotContains(response, self.p3.title)
	

class PostTestCase(BaseTestCase):
	
	
	def test_index_empty(self):
		response = self.client.get(reverse('mesh_post_index'))
		
		self.assertEqual(response.status_code, 200)
		self.assertContains(response, "There are no posts to display.")
	
	def test_index_has_only_active_posts(self):
		self.c1.save()
		self.p1.save()
		self.p1.channels.add(self.c1)
		self.p1.save()
		self.p2.save()
		self.p2.channels.add(self.c1)
		self.p2.save()
		self.p3.save()
		self.p3.channels.add(self.c1)
		self.p3.save()
		
		response = self.client.get(reverse('mesh_post_index'))
		
		self.assertEqual(response.status_code, 200)
		self.assertContains(response, self.p1.title)
		self.assertNotContains(response, self.p2.title)
		self.assertNotContains(response, self.p3.title)
	
	def test_view(self):
		self.c1.save()
		self.p1.save()
		self.i1.post = self.p1
		self.i1.save()
		self.comment1.content_object = self.p1
		self.comment1.save()
		self.p2.save()
		self.i2.post = self.p2
		self.i2.save()
		
		response = self.client.get(reverse('mesh_post_view', args=[self.p1.slug]))
		
		self.assertEqual(response.status_code, 200)
		self.assertContains(response, self.p1.title)
		self.assertContains(response, self.i1.title)
		self.assertContains(response, self.comment1.comment)
		self.assertNotContains(response, self.p2.title)
		self.assertNotContains(response, self.i2.title)
	
	def test_comments(self):
		self.c1.save()
		self.p1.save()
		self.i1.post = self.p1
		self.i1.save()
		self.comment1.content_object = self.p1
		self.comment1.save()
		self.p2.save()
		self.i2.post = self.p2
		self.i2.save()
		
		response = self.client.get(reverse('mesh_post_comments', args=[self.p1.slug]))
		
		self.assertEqual(response.status_code, 200)
		self.assertContains(response, self.p1.title)
		self.assertNotContains(response, self.i1.title)
		self.assertContains(response, self.comment1.comment)
		self.assertNotContains(response, self.p2.title)
		self.assertNotContains(response, self.i2.title)
	
	
	



# -*- coding: utf-8 -*-
#Copyright (C) 2011 Seán Hayes
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.

from ..models import *
from util import BaseTestCase

class TestPost(BaseTestCase):
	def test_has_auto_summary(self):
		self.c1.save()
		self.p1.save()
		self.p1.channels.add(self.c1)
		self.p1.save()
		
		assert self.p1.summary == self.p1.teaser
		assert self.p1.summary != self.p1.custom_summary
	
	def test_has_custom_summary(self):
		self.c1.save()
		self.p1.custom_summary = 'This is a summary.'
		self.p1.save()
		self.p1.channels.add(self.c1)
		self.p1.save()
		
		assert self.p1.summary != self.p1.teaser
		assert self.p1.summary == self.p1.custom_summary




#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup
import django_mesh

setup(name='django-mesh',
	version=django_mesh.__version__,
	description="A Django blog.",
	author='Seán Hayes',
	author_email='sean@seanhayes.name',
	classifiers=[
		"Development Status :: 3 - Alpha",
		"Framework :: Django",
		"Intended Audience :: Developers",
		"License :: OSI Approved :: GNU General Public License (GPL)",
		"Operating System :: OS Independent",
		"Programming Language :: Python",
		"Programming Language :: Python :: 2.6",
		"Topic :: Internet :: WWW/HTTP :: Dynamic Content",
		"Topic :: Internet :: WWW/HTTP :: Site Management",
		"Topic :: Software Development :: Libraries",
		"Topic :: Software Development :: Libraries :: Python Modules"
	],
	keywords='django blog',
	url='https://github.com/SeanHayes/django-mesh',
	license='GPL',
	packages=['django_mesh', 'django_mesh.tests',],
	install_requires=['Django>=1.2', 'django-pagination',],
)




# -*- coding: utf-8 -*-
#Copyright (C) 2011 Seán Hayes
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.contrib import admin
from models import *

class ChannelAdmin(admin.ModelAdmin):
	prepopulated_fields = {"slug": ("name",)}
admin.site.register(Channel, ChannelAdmin)

class ItemInlineAdmin(admin.TabularInline):
	model = Item
	extra = 0

class PostAdmin(admin.ModelAdmin):
	prepopulated_fields = {"slug": ("title",)}
	inlines = [ItemInlineAdmin]
	list_editable = ['status']
	list_display = ('title', 'author', 'status', 'last_edited', 'published',)
	list_filter = ('author', 'status', 'channels', 'created', 'last_edited', 'published',)
	readonly_fields = ('created', 'last_edited',)
	search_fields = ['title', 'text']
	
	def formfield_for_foreignkey(self, db_field, request, **kwargs):
		if db_field.name == 'author':
			db_field.default = request.user
		return super(PostAdmin, self).formfield_for_foreignkey(db_field, request, **kwargs)

admin.site.register(Post, PostAdmin)

class ItemAdmin(admin.ModelAdmin):
	list_display = ('title', 'post',)
	search_fields = ['title', 'text']

admin.site.register(Item, ItemAdmin)



# -*- coding: utf-8 -*-
#Copyright (C) 2011 Seán Hayes
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.

from ..models import *
from util import BaseTestCase

class TestPostManager(BaseTestCase):
	def test_active(self):
		self.p1.save()
		self.p2.save()
		self.p3.save()
		
		active_posts = Post.objects.active()
		
		assert self.p1 in active_posts
		assert self.p2 not in active_posts
		assert self.p3 not in active_posts




# -*- coding: utf-8 -*-
#Copyright (C) 2011 Seán Hayes
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.conf.urls.defaults import *

# place app url patterns here
urlpatterns = patterns('django_mesh.views',
	url(r'^$', 'index', name="mesh_index"),
	url(r'^channels/$', 'channel_index', name="mesh_channel_index"),
	url(r'^channels/(.*)/$', 'channel_view', name="mesh_channel_view"),
	url(r'^posts/$', 'post_index', name="mesh_post_index"),
	url(r'^posts/(.*)/comments/$', 'post_comments', name="mesh_post_comments"),
	url(r'^posts/(.*)/$', 'post_view', name="mesh_post_view"),
)
